;;;
;;;; Utility Functions
;;;
;;
;; @created   "Mon Oct 09 09:41:08 BST 2006"
;;

(require 'common-list-functions)

(declare
    (standard-bindings)
    (extended-bindings)
    (block))

(define (member-by-pred x l p)
  (cond  ((null? l)
	  #f)
	  ((p x (car l)) 
	   l)
          (else (member-by-pred x (cdr l) p))))

(define (mixup pool . extract)
  (declare (fixnum) )
  (letrec ((mixup (lambda(pool new-pool len)
                    (if (= len 0)
                        new-pool
                        (let* ((l len)
                               (s (random-integer (length pool)))
                               (hd (car (nthcdr s pool))))
                          (mixup (delete-nth-elt s pool) (cons hd new-pool) (- len 1)))))))
    (mixup pool '() (if (null? extract)
			(length pool)
			(car extract)))))



(define (delete-nth-elt n l)
  (declare (fixnum) )
  (nconc (butnthcdr n l)
         (cdr (nthcdr n l))))

  
(define (nth n l) (car (nthcdr n l)))

(define (make-integer-list n)
  (declare (fixnum) )
  ;; Return the first n integers starting from 0
  (mapn (lambda(i)i) n))

(define (zip a b)
  (if (null? a)
      '()
      (cons (cons (car a) (car b)) (zip (cdr a) (cdr b)))))

(define (lzip a b)
  (if (null? a)
      '()
      (cons (list (car a) (car b)) (lzip (cdr a) (cdr b)))))


(define (collect items pred?)
  (let ((res (list '*HEAD*)))
    (let loop ((tl res)
	       (il items))
      (cond
       ((null? il) (cdr res))
       ((pred? (car il)) 
	(set-cdr! tl (list (car il)))
	(loop (cdr tl) (cdr il)))
       (else
	(loop tl (cdr il)))))))
         

(define (mapcdr f l)
  ;; Map down the cdrs of the list applying f to each cdr
  ;; in turn
  (if (null? l)
      '()
      (cons (f l)
            (mapcdr f (cdr l)))))

(define (set-cadr! l v)
  (set-car! (cdr l) v))

(define (mod x1 x2)
  ;; This should be defined in SLIB but for some reasons it's disappeared
   (- x1 (* x2 (floor (/ x1 x2 )))))

(define (nth-items indexes items)
  ;; Return the entries in items indexed by indexes
  ;; which must be an integer-list in increasing order
  (declare (fixnum))
  (letrec ((item-list (lambda(i l j)
			(cond 
			 ((null? i) '())
			 ((null? l) (error "index exceeded list" indexes list))  
			 ((= j (car i))
			  (cons (car l)
				(item-list (cdr i) (cdr l) (+ 1 j))))
			 (else (item-list i (cdr l) (+ 1 j)))))))
    (item-list indexes items 0)))
			    

(define (nthcdr-items index indices)
  ;; Return the indices following indice index 
  ;; e.g 3 (2 3 5 9) =p=> (3 5 9)
  (declare (fixnum))
  (cond
   ((null? indices) '())
   ((= index (car indices)) indices)
   ((< index (car indices)) indices)
   (else (nthcdr-items index (cdr indices)))))

(define (butnthcdr-items index indices)
  ;; Return the indices before indice index 
  ;; e.g 3 (2 3 5 9) =p=> (3 5 9)
  (declare (fixnum))
  (cond
   ((null? indices) '())
   ((= index (car indices)) '())
   ((< index (car indices)) '())
   (else (cons (car indices) 
	       (butnthcdr-items index (cdr indices))))))

(define (long-directory-list d)
	(map (lambda(f)(string-append d "/" f))
             (directory-files d)))

(define (map-vector f v)
  (declare  (fixnum))
  (let ((l (vector-length v)))
    (let loop ((i l)
               (r '()))
      (cond ((= i 0) r)
            (else (loop 
                     (- i 1)
                     (cons (f (vector-ref v (- i 1)))  r)))))))

(define (vector-some f v)
  (declare  (fixnum))
  (let ((l (vector-length v)))
    (let loop ((i l))
      (cond ((= i 0) #f)
	    ((f (vector-ref v (- i 1))) #t)
            (else (loop 
                     (- i 1)))))))

(define (safe-vector-ref v i)
  (if (< i (vector-length v))
      (vector-ref v i)
      #f))

(define (remove-first obj lst)
  (let  ((head (list '*head*)))
    (let remove ((lst lst)
		 (tail head))
      (cond ((null? lst))
	    ((eqv? obj (car lst)) (set-cdr! tail (cdr lst)))
	    (else
	     (set-cdr! tail (list (car lst)))
	     (remove (cdr lst) (cdr tail)))))
    (cdr head)))

(define (remove-if-present obj lst)
  (if (member obj lst)
      (remove obj lst)
      lst))

(define (number-and-zip l)
  ;; Create a zipped list of the elements of l with their position in l
  (declare (fixnum))
  (letrec ((zip (lambda(l p)(if (null? l) 
			     '()
			     (cons (cons p (car l))
				   (zip (cdr l) (+ 1 p)))))))
    (zip l 0)))

(define (locations-of f l)
  (declare (fixnum))
  ;; Return a list of indexes in list l where f is true
  (letrec ((loc-of (lambda  (i l) (cond 
				   ((null? l) '())
				   (else 
				    (if (f (car l)) 
					(cons i (loc-of (+ 1 i) (cdr l)))
					(loc-of (+ 1 i) (cdr l))))))))
    (loc-of 0 l)))

(define (nassoc e l)
  ;; A numeric assoc assuming keys in increasing order and 
  ;; possibly non-contigious
  (declare (fixnum))
  (cond
   ((null? l) 0.0)
   ((< e (caar l)) 0.0)
   ((> e (caar l)) (nassoc e (cdr l)))
   (else (cdar l))))


(define (assoc-def e l d)
  ;; Assoc but with a default
  (let ((a (assq e l)))
    (if a
	(cdr a)
	d)))


(define (make-sparse l)
  (if (not (list? l))
      l
      (cons 'SPARSE (reduce-init (lambda(left-total right-next) 
				   (let ((p (car left-total))
					 (l (cdr left-total)))
				     (cond ((list? right-next) 
					    (cons (+ 1 p) (append l (list (cons p (make-sparse right-next))))))
					   ((and (number? right-next) ( = right-next 0))
					    (cons (+ 1 p) l))
					   (else 
					    (cons (+ 1 p) (append l (list (cons p right-next))))))))
				 (cons 0 '())
				 l))))

(define (de-sparse l)
  (cond
   ((null? l) 
    l)
   ((not (list? l))
    l)
   ((not (eq? 'SPARSE (car l)))
    (map de-sparse l))
   (else
    (let ((len (cadr l))
	  (contents (cddr l)))
      (letrec ((de-sp (lambda (l p)
			(cond ((null? l) 
			       '())
			      ((and (= p (caar l)) (not (list? (cdar l))))
			       (cons (cdar l) (de-sp (cdr l) (+ 1 p))))
			      ((= p (caar l))
			       (cons (de-sparse (cdar l)) (de-sp (cdr l) (+ 1 p))))
			      (else (cons 0 (de-sp l (+ 1 p))))))))
	(let ((dsl (de-sp contents 0 )))
	  (if (= (length dsl) len)
	      dsl
	      (append dsl (make-list (- len (length dsl)) 0)))))))))

(define *log* #f)

(define *termite-log* #t)

(define (log-on-off)
   (set! *log* (not *log*)))

(define (termite-log-message m . v)
  (cond 
   (*termite-log* 
     (display (now))
     (display ": ")
     (display m)
     (map (lambda(v) (display " ") (if (string? v) (display v) (write v))) v)
     (newline)))
  #t)


(define (log-message m . v)
  (cond 
   (*log* 
     (display (now))
     (display ": ")
     (display m)
     (map (lambda(v) (display " ") (if (string? v) (display v) (write v))) v)
     (newline)))
  #t)

(define (mapn f n)
  (let ((l (list '*head*)))
    (let loop ((i 0)
	       (tl l))
      (cond ((>= i n) (cdr l))
	    (else (set-cdr! tl (list (f i)))
		  (loop (+ 1 i) (cdr tl)))))))

(define (mapn-if f n)
  (let ((l (list '*head*)))
    (let loop ((i 0)
	       (tl l))
      (cond ((>= i n) (cdr l))
	    (else 
	     (let ((r (f i)))
	       (cond
		(r 
		 (set-cdr! tl (list r))
		 (loop (+ 1 i) (cdr tl))) 
		(else
		 (loop (+ 1 i) tl)))))))))


(define (map-if f . args)
  (let ((l (list '*head*)))
    (let loop ((i args)
	       (tl l))
      (cond ((every null? i) (cdr l))
	    (else 
	     (let ((r (apply f (map car i))))
	       (cond
		(r 
		 (set-cdr! tl (list r))
		 (loop (map cdr i) (cdr tl))) 
		(else
		 (loop (map cdr i) tl)))))))))

(define (count x l)
  (declare (fixnum))
  (reduce-init
   (lambda(total next)(if (equal? next x)(+ total 1) total))
   0
   l))

(define (count-if? x l)
  (declare (fixnum))
  (reduce-init
   (lambda(total next)(if (x next)(+ total 1) total))
   0
   l))

(define (map-unique f l)
  ;; Yield the equivalent of (remove-duplicates (map f l))
  (let ((res (list '*HEAD*)))
  (let loop ((tl res)
	     (xl l))
    (cond 
     ((null? xl) (cdr res))
     (else
      (let ((fx (f (car xl))))
	(cond
	 ((member fx (cdr res))
	    (loop tl (cdr xl)))
	 (else
	  (set-cdr! tl (list fx))
	  (loop (cdr tl) (cdr xl))))))))))


(define (reduce-map fReduce fMap init . args)
  ;; Yield the equaivalent of (reduce fReduce (map fMap args))
  (let loop ((res init)
	     (xl args))
    (cond 
     ((every null? xl) res)
     (else
      (let ((fx (apply fMap (map car xl))))
	(set! res (fReduce res fx))
	(loop res (map cdr xl)))))))

(define (reduce-until fTerminate fReduce l init)
  (let loop ((res init)
	     (xl l))
    (cond 
     ((null? xl) (values res xl))
     ((fTerminate res (car xl)) (values res xl))
     (else (loop (fReduce res (car xl)) (cdr xl))))))
	   

(define (maploop f . l)
  ;; Same as map except no list is built, so useful for side effects 
  ;; only
  (let loop ((args l))
    (cond 
     ((not (every null? args))
	(apply f (map car args))
	(loop (map cdr args))))))

(define (mapnloop f n)
  ;; Same as mapn except no list is built, so useful for side effects 
  ;; only
  (let loop ((i 0))
      (cond ((< i n)
	     (f i)
	     (loop (+ 1 i))))))
