;;; Termite code to contact mast node and inform that I am available 
;;; for use as a processing node

(declare (block))

(define slave-node '())

(define (slaveme-to host master . port)
  (let ((port (if (null? port) 8888 (car port)))
	(master-node (make-node master 7777)))
    (set! slave-node (make-node host port))
    (node-init slave-node)
    (let ((me (self))
	  (tag (make-tag)))
      (termite-log-message ";; Slaving to master server " master-node)
      (remote-spawn master-node
		    (lambda () (with-exception-catcher
				(lambda(e) (display (list 'error (object->string (handler e)))))
				(lambda()(assign-slave host port)))))
      ;; Now request the master to clone its server onto this slave
      (termite-log-message ";; Requesting clone from master " master-node)
      (remote-spawn master-node
		    (lambda () (with-exception-catcher
				(lambda(e) (display (list 'error (object->string (handler e)))))
				(lambda()(clone-server me tag)))))
      (recv 
       ((,tag clone)
	(termite-log-message ";; Received clone tagged " (list tag clone))
	(set! slave-pid-server 
	      (remote-spawn slave-node (lambda()(termite-log-message "Starting clone")(clone)))))
       (after 10 (error "Timeout")))
	  
      (if (null? slave-pid-server) (error "Null slave pid server"))

      ;; Tell the master that it now cloned to this slave 
      ;; so the master will in  future inform it of new slaves
      (termite-log-message ";; Informing master slave is now cloned " slave-pid-server)
      (remote-spawn master-node
 		    (lambda () (with-exception-catcher
 				(lambda(e) (display (list 'error (object->string (handler e)))))
 				(lambda () (server-is-cloned slave-pid-server host port)))))

      (termite-log-message ";; Slavery is now complete " slave-node))))
		    
(define (unslave host master . port)
  (let ((port (if (null? port) 8888 (car port)))
	(master-node (make-node master 7777)))
    (! master-node (list 'remove-slave host port))))
  
